Maq 0.7.1 for Windows
=========================

Source Code Modifications
-------------------------

Maq was designed for compilation under UNIX/POSIX-compliant systems.  To compile it under Windows, some changes to its source code were made:

1.	In `stdhash.hh`, on lines 415 and 496, calls to `direct_insert_aux()` were replaced
	with calls to `this->direct_insert_aux()` to resolve a compile-time ambiguity about
	which method to call in the templated subclass `hash_set_misc`.
2.	In `match.cc`, the `match_search()` function used the `getrusage()` function,
	declared on UNIX-style systems in `<sys/resource.h>`, to time its matching
	operation.  The call to `getrusage()` was replaced with a call to `get_process_seconds()`,
	declared in `lib/process_secs.h`, and implemented in `lib/process_secs.c`
	using `getrusage()` on UNIX (when `HAVE_SYS_RESOURCE_H` is defined)
	and using `GetProcessTimes()` on Windows.  A `Makefile.am`
	was added to the `lib` subdirectory, and `configure.ac` was modified to check
	for the presence of `<sys/resource.h>` and to build and link in
	a compatibility library, `lib/libcompat.a`, containing the `get_process_seconds()` function.
3.	In `fastq2bfq.c`, `#include <stdint.h>` was added on MinGW.  This header declares
	the fixed-width integer types used in the file.
4.	`fasta2bfa.c` and `algo.hh` use the `srand48()`, `erand48()`, and `drand48()` functions
	to generate random double-precision floating-point numbers.  These functions are not
	implemented in the MinGW-w64 C runtime.  Implementations, adapted from FreeBSD,
	were added, comprising the files `lib/drand48.h` and `lib/drand48.c`.  The following conditional inclusion
	was added to both `fasta2bfa.c` and `algo.hh`.
	
		#if defined(__MINGW64__) || defined(__MINGW32__)
		#include "lib/drand48.h"
		#endif
	
	Finally, `configure.ac`, `Makefile.am`, and `lib/Makefile.am` were modified to
	conditionally compile and link the `drand48` algorithms if they are not present
	in system libraries.  This is achieved through the following autoconf commands:
	
		AC_CONFIG_LIBOBJ_DIR([lib])
		AC_REPLACE_FUNCS([drand48])
		
5.	On Windows, binary files opened in text mode can become corrupted due to line-ending
	conversion and truncation at the EOF control character.  As this is not an issue on UNIX,
	Maq did not explicitly open its binary FastA files in binary mode.  Binary mode flags were
	added to `fopen` for all binary FastA files.  This change affected the following files:
	
		altchr.cc
		assopt.c
		fasta2bfa.c
		indel_call.cc
		indel_soa.cc
		mapcheck.cc
		match.cc
		pair_stat.cc
		pileup.cc
	
	Note that no change had to be made for binary FastQ files, as they are compressed
	with GZip, and `zlib` handles their I/O transparently.
